/*
 * Copyright E. Bigeon (2015)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a swt window for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-swt:fr.bigeon.gclc.swt.HistoryTextKeyListener.java
 * Created on: Jun 9, 2016
 */
package fr.bigeon.gclc.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Text;

import fr.bigeon.collections.ArrayRibbon;
import fr.bigeon.collections.Ribbon;

/** A key listener to validate commands and manage the history of commands
 *
 * @author Emmanuel Bigeon */
public final class HistoryTextKeyListener extends KeyAdapter {

    /** The size of commands history */
    private static final int DEFAULT_HISTORY_SIZE = 10;
    /** The history ribbon */
    private final Ribbon<String> commands;
    /** The current index in history search */
    private int currentIndex = 0;
    /** The console to write the commands in */
    private final Text consoleInput;
    /** The console to notify of command validation */
    private final SWTConsole console;

    /** @param console the console
     * @param consoleInput the text to write commands in */
    public HistoryTextKeyListener(SWTConsole console, Text consoleInput) {
        super();
        this.console = console;
        this.consoleInput = consoleInput;
        this.commands = new ArrayRibbon<>(DEFAULT_HISTORY_SIZE);
    }

    @Override
    public void keyPressed(KeyEvent e) {

        pressedKeyCode(e.keyCode);
    }

    /** @param keyCode */
    public void pressedKeyCode(int keyCode) {
        // Enter validates the command if prompting
        if (keyCode == '\r') {
            commands.add(consoleInput.getText());
            console.validateInput();
            currentIndex = -1;
        }

        // Upper arrow retrieves previous commands
        if (keyCode == SWT.ARROW_UP &&
            currentIndex < commands.size() - 1) {
            currentIndex++;
            String cmd = commands.get(commands.size() - currentIndex - 1);
            consoleInput.setText(cmd);
            consoleInput.setSelection(cmd.length());
        }

        // Lower arrow retrieves next commands
        if (keyCode == SWT.ARROW_DOWN) {
            if (currentIndex == 0) {
                currentIndex--;
                consoleInput.setText(new String());
            } else if (currentIndex > 0) {
                String cmd = commands
                        .get(commands.size() - (--currentIndex) - 1);
                consoleInput.setText(cmd);
                consoleInput.setSelection(cmd.length());
            }
        }
    }
}