/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.CommandTestingApplication.java
 * Created on: Jun 12, 2016
 */
package fr.bigeon.gclc;

import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintWriter;

import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.SystemConsoleManager;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon
 *
 */
public class CommandTestingApplication {

    private final PrintWriter writer;
    private final ConsoleTestApplication application;
    private final Thread th;

    /** @throws IOException if the streams cannot be build */
    public CommandTestingApplication() throws IOException {
        final PipedOutputStream src = new PipedOutputStream();
        InputStream in = new PipedInputStream(src);

        application = new ConsoleTestApplication(
                new SystemConsoleManager(System.out, in));
        th = new Thread(new Runnable() {

            @Override
            public void run() {
                application.start();
            }
        });

        th.start();
        writer = new PrintWriter(src, true);
    }

    public void stop() {
        application.exit();
        writer.println();
    }

    public void sendCommand(String cmd) {
        writer.println(cmd);
    }

    /** @param cmd the command
     * @return if the command was added
     * @throws InvalidCommandName if the command name is invalid
     * @see fr.bigeon.gclc.ConsoleApplication#add(fr.bigeon.gclc.command.ICommand) */
    public final boolean add(ICommand cmd) throws InvalidCommandName {
        return application.add(cmd);
    }

    /** @return the application */
    public ConsoleTestApplication getApplication() {
        return application;
    }

    /**
     * 
     */
    public void waitAndStop() {
        writer.println(ConsoleTestApplication.EXIT);
        try {
            th.join();
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
